@@ -69,6 +69,7 @@ impl ArchetypeDescriptor {
     }
 
     /// Returns whether the descriptor provided is contained in self. (i.e. subset inclusion)
+    /// Do not provide an invalid descriptor to this!
     pub const fn contains_subset(&self, descriptor: &ArchetypeDescriptor) -> bool {
         if descriptor.len() > self.len() {
             return false;
@@ -92,6 +93,27 @@ impl ArchetypeDescriptor {
         return true;
     }
 
+    /// Returns whether the descriptor provided is excluded from self. (i.e. subset exclusion)
+    /// Do not provide an invalid descriptor to this!
+    pub const fn excludes_subset(&self, descriptor: &ArchetypeDescriptor) -> bool {
+        let mut i = 0;
+        while i < descriptor.len() {
+            let mut j = 0;
+            while j < self.len() {
+                if self.components[j as usize].component_type_id.into_u16()
+                    == descriptor.components[i as usize]
+                    .component_type_id
+                    .into_u16()
+                {
+                    return false;
+                }
+                j += 1;
+            }
+            i += 1;
+        }
+        return true;
+    }
+
     /// Returns a new archetype with the given component type added to it.
     /// Returns none if the current archetype already contains the component type or it is full.
     #[allow(dead_code)]
@@ -154,7 +176,6 @@ impl ArchetypeDescriptor {
     }
 
     /// Returns whether the archetype descriptor has a given component type.
-    // TODO: Check if this can be constified somehow?
     pub fn has_component<C: Component>(&self) -> bool {
         return match self
             .components()
@@ -177,6 +198,7 @@ impl ArchetypeDescriptor {
 
     /// Get a reference to the archetype descriptor's components.
     /// This version is const but unsafe, as length is NOT accounted for.
+    /// Therefore any descriptor past the len is considered invalid or garbage data.
     pub const unsafe fn components_unchecked(
         &self,
     ) -> &[ComponentDescriptor; MAX_COMPONENTS_PER_ENTITY] {
@@ -235,4 +257,26 @@ mod tests {
             false
         );
     }
+
+    #[test]
+    fn test_archetype_descriptor_excludes() {
+        assert_eq!(
+            <(A, B) as ComponentGroup>::DESCRIPTOR
+                .archetype()
+                .excludes_subset(<C as ComponentGroup>::DESCRIPTOR.archetype()),
+            true
+        );
+        assert_eq!(
+            <(A, B) as ComponentGroup>::DESCRIPTOR
+                .archetype()
+                .excludes_subset(<A as ComponentGroup>::DESCRIPTOR.archetype()),
+            false
+        );
+        assert_eq!(
+            <(A, B) as ComponentGroup>::DESCRIPTOR
+                .archetype()
+                .contains_subset(<B as ComponentGroup>::DESCRIPTOR.archetype()),
+            true
+        );
+    }
 }
