@@ -17,4 +17,7 @@ pub use registry::Registry;
 pub use archetype::Archetype;
 pub use entity_registry::Entity;
 pub use descriptors::*;
-pub use constants::*;
\ No newline at end of file
+pub use constants::*;
+
+#[cfg(feature = "derive")]
+pub use shard_ecs_derive::*;
\ No newline at end of file
